#!/usr/bin/env python
import pyglet
pyglet.resource.path.append("art")
pyglet.resource.path.append("art/vegetables")
pyglet.resource.path.append("art/hands")
pyglet.resource.reindex()

from pyglet.window import key
from pubsub import Publisher
import playstate, splashstate, constants


class Window(pyglet.window.Window):
    def __init__(self):
        pyglet.window.Window.__init__(self, width=800, height=600, caption="Urban Biodynamics v%s" % constants.VERSION)
        keys = key.KeyStateHandler()
        self.push_handlers(keys)
        #self.fps = pyglet.clock.ClockDisplay()
        pyglet.clock.schedule_interval(self.tick, 1/30.0)
        
        self.State = splashstate.SplashState(self)
        Publisher.subscribe(self.onGameNew, "game.new")
        Publisher.subscribe(self.onGameNew, "game.continue")
        Publisher.subscribe(self.onGameExit, "game.exit")
        
    def onGameNew(self, message):
        self.State.unsubscribe()
        self.State = playstate.PlayState(self)
        self.State.start()
        
    def onGameContinue(self, message):
        pass
        
    def onGameExit(self, message):
        import sys; sys.exit()
        
    def on_key_press(self, symbol, modifiers):
        Publisher.sendMessage("keypress.%s" % symbol, modifiers)

    def tick(self, dt):
        self.State.tick(dt)
        
    def on_draw(self):
        self.clear()
        self.State.draw()
        #self.fps.draw()
        
    def on_mouse_motion(self, x, y, dx, dy):
        if x < 600 and y < self.height:
            tx = (x) // constants.TILE_SIZE
            ty = (y) // constants.TILE_SIZE
            Publisher.sendMessage("tile.highlighted", (tx, ty))
            
    def on_mouse_press(self, x, y, button, mods):
        if button == pyglet.window.mouse.LEFT:
            Publisher.sendMessage("click.left")

        
def main():
    w = Window()
    pyglet.app.run()

if __name__ == "__main__":
    main()
