import pyglet, random
import items, utils, plants, constants
from pubsub import Publisher



class Map(utils.Subscribable):
    def __init__(self, window):
        self.Highlighted = None
        
        height = 0
        self.Rows = []
        while height < window.height: # - Tile(0,0).height:
            row = TileRow(window, height)
            self.Rows.append(row)
            height += row.height
            
        self.Subscriptions = (
            (self.onHighlight, "tile.highlighted"),
            (self.onItemDropping, "item.dropping"),
        )
        self.subscribe()
        
    def clear(self):
        self.unsubscribe()
        for tilerow in self.Rows:
            for tile in tilerow.Tiles:
                tile.clear()
        
    def onItemDropping(self, message):
        item = message.data
        if self.Highlighted:
            dropped = self.Highlighted.acceptItem(item)
            if dropped:
                Publisher.sendMessage("item.dropped", item)
    
    def onHighlight(self, message):
        self.highlight(*message.data)
        
    def highlight(self, x, y):
        self.unhighlight()
        tile = self.Rows[y].Tiles[x]
        tile.highlight()
        self.Highlighted = tile
        
    def unhighlight(self):
        if self.Highlighted:
            self.Highlighted.unhighlight()
            self.Highlighted = None
            
    def tick(self, dt):
        for row in self.Rows:
            row.tick(dt)
            
    def draw(self, parent):
        constants.tilebatch.draw()

class TileRow(object):
    def __init__(self, parent, y):
        width = 0
        self.Tiles = []
        while width < parent.width:
            tile = Tile(width, y)
            self.Tiles.append(tile)
            width += tile.width
            
        self.height = tile.height
            
    def tick(self, dt):
        for tile in self.Tiles:
            tile.tick(dt)
            
    y = property(lambda s: s.Tiles[0].y)

class Tile(items.BaseItem):
    IMAGE = pyglet.resource.image("tile1.png")
    def __init__(self, x, y):
        items.BaseItem.__init__(self)
        self.position = (x, y)
        self.Items = []
        self.addItem(self)
        
    def clear(self):
        items.BaseItem.clear(self)
        for item in self.Items[1:]:
            item.clear()
        
    def acceptItem(self, item):
        if len(self.Items) == 1:
            inst = item()
            if isinstance(inst, plants.BasePlant):
                self.addItem(inst)
                return True
            else:
                return False
        else:
            self.Items[-1].acceptItem(item)
        
    def addItem(self, item):
        item.position = self.position
        item.batch = constants.tilebatch
        item.group = constants.layergroups[len(self.Items)]
        self.Items.append(item)
        
    def tick(self, dt):
        for item in self.Items[1:]:
            item.tick(dt)
        
    def highlight(self):
        self.opacity = 255 * 0.8
        
    def unhighlight(self):
        self.opacity = 255
        